#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/io.h>
#include <stdlib.h>

#include "IOPort.h"
#include "AtoiHex.h"

unsigned long ParPort_Base;
unsigned int RegValue_Control_Before;

int Getch(void)
{
  int ch;
  struct termios oldt,newt;

  tcgetattr(STDIN_FILENO, &oldt);
  newt = oldt;
  newt.c_lflag &= ~(ECHO|ICANON);
  tcsetattr(STDIN_FILENO, TCSANOW, &newt);
  ch = getchar();
  tcsetattr(STDIN_FILENO, TCSANOW, &oldt);

  return ch;
}

/* SET DISPLAY COMMANDS 1-8 */
void SetCommand(unsigned int RegValue_Data)
{
	unsigned int RegValue_Control;
	
	while(Out_Port(ParPort_Base, 0x00, RegValue_Data) !=1);
	
	RegValue_Control = 0x02 + (RegValue_Control_Before & 0xf0);
	while(Out_Port(ParPort_Base, 0x02, RegValue_Control) !=1);
	
	RegValue_Control = 0x03 + (RegValue_Control_Before & 0xf0);
	while(Out_Port(ParPort_Base, 0x02, RegValue_Control) !=1);
		  usleep(40); // Execution Time 40us
}

/* SET DISPLAY COMMANDS 10 */
void WriteCommand(unsigned int RegValue_Data)
{
	unsigned int RegValue_Control;
	
	while(Out_Port(ParPort_Base, 0x00, RegValue_Data) !=1);
	
	RegValue_Control = 0x06 + (RegValue_Control_Before & 0xf0);
	while(Out_Port(ParPort_Base, 0x02, RegValue_Control) !=1);
	
	RegValue_Control = 0x07 + (RegValue_Control_Before & 0xf0);
	while(Out_Port(ParPort_Base, 0x02, RegValue_Control) !=1);
		  usleep(40); // Execution Time 40us
}

void Initial(void)
{
	usleep(15000);
	SetCommand(0x38);
	usleep(5000);
	SetCommand(0x38);
	usleep(5000);
	SetCommand(0x38);
	SetCommand(0x38);
	SetCommand(0x08);
	SetCommand(0x06);
	SetCommand(0x0c);
	SetCommand(0x01);
	usleep(50000); 
}

void WriteData(int row, char data[])
{
	int i, Data_ASCII;
	
	Data_ASCII = 0;

	if (row == 0)
		SetCommand(0x80);
	else
		SetCommand(0xc0);
	
	usleep(1640); 	//;SHL-X001 Execution Time 1.64ms
	
	for(i = 0; data[i] != '\0'; i++)
	{
		if (data[i] >= 0x20 && data[i] <= 0x7f) 
			Data_ASCII = data[i];
		else
			Data_ASCII = 0x20;
			
		WriteCommand(Data_ASCII);	
	}
}

int main(int argc, char *argv[])
{
	if (argc < 2 || argv[1][0] == '?' || argc > 4)
	{
		printf("\n\033[7mCopyright (C) 2008 Advantech Company Design by Marlboro Version 1.0.5. (Updated by KMTD) \n");
		printf("LCM Module Print Out Program\033[0m\n\n");
		printf("\033[7mUSAGE :\033[0m\n");
		printf("	LCMPrint [ IO ADDRESS | String 0f Line 01 | String 0f Line 02 ]\n\n");
		printf("\033[7mEXAMPLE :\033[0m\n");
		printf("	Set LCM Initially:             LCMPrint 378\n");
		printf("	Only Print Line 01:            LCMPrint 378 'String_Line_01'\n");
		printf("	Only Print Line 02:            LCMPrint 378 '' 'String_Line_02'\n");
		printf("	Both Print Line 01 and 02:     LCMPrint 378 'String_Line_01' 'String_Line_02'\n\n");
		return 0;
	}
	
	ParPort_Base = atoihex(argv[1]);
	
	while(IN_Port(ParPort_Base, 0x02, &RegValue_Control_Before) !=1);
	
	Initial();

			
	if (argc > 2)
		WriteData(0, argv[2]);
	if (argc > 3)
		WriteData(1, argv[3]);
	
	return 0;
}
